#ifndef GAR3D_CLASS_HPP
#define GAR3D_CLASS_HPP

#include <iostream>
#include <string>
#include <vector>
#include "SDL_GLM_INCLUDES.hpp"
#include "InputHandle.hpp"
#include "camera.hpp"
#include "Time/Time.h"
#include "Color.hpp"

namespace gar3d {
	
	enum orientation : int {
		positive_x,
		positive_y,
		positive_z,
		negative_x,
		negative_y,
		negative_z,
	};

	enum primitive : int {
		sphere,
		box,
		cylinder,
		cone,
		pyramid_3,
		pyramid_4,
		pyramid_6,
		plane,
		empty,
		arrow,
		line,
	};

	enum cameraMode : int {
		pancam,
		orbitcam,
		freecam,
	};

	class Gar3D {
	public:
		Gar3D();
		Gar3D(int width, int height);
		~Gar3D();
		// drawing
		void drawPrimitive(primitive prim, orientation orien, double posx, double posy, double posz, double size);
		void drawPrimitive(primitive prim, orientation orien, double posx, double posy, double posz, double sizex, double sizey, double sizez);
		void lineSize(double size) { glLineWidth(size); }
		void drawLine(double x0, double y0, double z0, double x1, double y1, double z1);
		void renderFrame();
		// camera
		void getFreeCameraPosition(double& x, double& y, double& z);
		void setFreeCameraPosition(double x, double y, double z);
		void getFreeCameraTarget(double& x, double& y, double& z);
		void setFreeCameraTarget(double x, double y, double z);
		void getPanCameraMinBounds(double& x, double& y, double& z);
		void setPanCameraMinBounds(double x, double y, double z);
		void getPanCameraMaxBounds(double& x, double& y, double& z);
		void setPanCameraMaxBounds(double x, double y, double z);
		void getActiveCameraPosition(double& x, double& y, double& z);
		cameraMode CameraMode();
		cameraMode CameraMode(cameraMode mode);
		// light
		double lightBrightness() { return _brightness; }
		double lightBrightness(double brightness) { return _brightness = brightness; }
		// colors
		Color backgroundColor() { return _backgroundColor; }
		Color backgroundColor(Color col) { return _backgroundColor = col; }
		Color backgroundColor(ColorChannel r, ColorChannel g, ColorChannel b) { return _backgroundColor = Color(r, g, b); }
		Color color() { return _objColor; }
		Color color(Color col) { return _objColor = col; }
		Color color(ColorChannel r, ColorChannel g, ColorChannel b) { return _objColor = Color(r, g, b); }
		// timing
		double deltatime() const { return _deltatime; }
		// input
		bool shouldQuit();
		const InputHandle& input() { return _input; }

	private:
		Color _backgroundColor;
		Color _objColor;
		SDL_Window *window;
		Timepoint _lastFrame;
		double _lsize;
		double _brightness;
		std::vector<primitive> _objbuffer;
		std::vector<orientation> _oribuffer;
		std::vector<vec3> _locbuffer;
		std::vector<vec3> _sizbuffer;
		std::vector<Color> _colbuffer;
		std::vector<vec3> _linebuffer;
		std::vector<Color> _linecolbuffer;
		Camera* _camera;
		OrbitCamera _orbitcam;
		PanCamera _pancam;
		FreeCamera _freecam;
		InputHandle _input;
		double _deltatime;
		void renderPrimitive(primitive prim, orientation orien, vec3 location, vec3 size, Color color);
		void addLight(GLenum light, glm::vec4 position, vec3 color);
		void handleInput();
		void renderSphere();
		void renderCube();
		void renderCylinder();
		void renderPyramid(int sides);
		void renderPlane();
		void renderArrow(float thickness);
		void renderLine();
		void renderCircle(int segments);
	};

}

#endif // GAR3D_CLASS_HPP
